package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.io.IoUtil;
import cn.hutool.core.io.file.FileReader;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.sun.mail.imap.IMAPFolder;
import org.apache.commons.lang3.StringUtils;
import pps.cloud.base.service.IBaseWeatherCloudService;
import pps.core.base.entity.BasePhotovoltaicPlantView;
import pps.core.base.entity.PlantPredictedPowerDataEnt;
import pps.core.base.enums.WindDirection;
import pps.core.base.mapper.BasePhotovoltaicPlantViewMapper;
import pps.core.base.mapper.PlantPredictedPowerDataMapper;
import pps.core.base.service.data.DailyData;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XServiceResult;

import javax.mail.*;
import javax.mail.internet.MimeUtility;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 天气数据接收Cloud模块
 *
 * @author ZWT
 * @date 2023/11/13 17:54
 */
@XService
public class BaseWeatherCloudServiceImpl implements IBaseWeatherCloudService {

    /**
     * 天气数据接收Cloud模块--天气数据处理
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult weatherDataProcess(XContext context) {
        String host = XStartup.getCurrent().getProperty("weather.mail.host");
        String username = XStartup.getCurrent().getProperty("weather.mail.username");
        String password = XStartup.getCurrent().getProperty("weather.mail.password");
        String protocol = XStartup.getCurrent().getProperty("weather.mail.protocol");
        //取未读邮件
        Store store = null;
        Folder folder = null;
        try {
            Properties props = System.getProperties();
            props.setProperty("mail.store.protocol", protocol);
            Session session = Session.getDefaultInstance(props, null);
            store = session.getStore(protocol);
            store.connect(host, username, password);
            folder = store.getFolder("INBOX");
            if (folder instanceof IMAPFolder) {
                IMAPFolder imapFolder = (IMAPFolder) folder;
                // javamail中使用id命令有校验checkOpened, 所以要去掉id方法中的checkOpened();
                imapFolder.doCommand(p -> {
                    Map<String, String> gmap = new HashMap<>();
                    gmap.put("GUID", "FUTONG");
                    p.id(gmap);
                    return null;
                });
            }
            if (Objects.nonNull(folder)) {
                folder.open(Folder.READ_WRITE);
            }
            Message[] messages = folder.getMessages();
            if (ArrayUtil.isNotEmpty(messages)) {
                List<Message> messageList = new ArrayList<>(16);
                for (Message message : messages) {
                    boolean isNew = false;
                    for (Flags.Flag flag : message.getFlags().getSystemFlags()) {
                        if (flag == Flags.Flag.SEEN) {
                            isNew = true;
                        }
                    }
                    if (!isNew && StrUtil.endWith(message.getSubject(), "天气预测数据")) {
                        messageList.add(message);
                    }
                }
                if (CollUtil.isEmpty(messageList)) {
                    return null;
                }
                //取电站
                BasePhotovoltaicPlantViewMapper viewMapper = context.getBean(BasePhotovoltaicPlantViewMapper.class);
                List<BasePhotovoltaicPlantView> plantList = viewMapper.selectPlantList();
                if (CollUtil.isEmpty(plantList)) {
                    return null;
                }
                //按照cityCode分组
                Map<String, List<BasePhotovoltaicPlantView>> plantMap = plantList.stream()
                        .collect(Collectors.groupingBy(BasePhotovoltaicPlantView::getCityCode));
                PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
                //读取附件
//                for (Message message : messageList) {
//                    String tempFilePath = this.saveAttachment(message);
//                    FileReader fileReader = new FileReader(tempFilePath);
//                    String jsonString = fileReader.readString();
//                    JSONObject jsonObject = JSONObject.parseObject(jsonString);
//                    Map<String, List<DailyData>> cityDataMap = new HashMap<>(16);
//                    for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
//                        String key = entry.getKey();
//                        Object value = entry.getValue();
//                        if (value instanceof JSONArray) {
//                            cityDataMap.put(key, ((JSONArray) value).toJavaList(DailyData.class));
//                        }
//                    }
//
//
//                    FileUtil.del(tempFilePath);
//                }
                FileReader fileReader = new FileReader("D:\\home\\weather\\temp\\receive\\20231114\\1699943400000.json");
                String jsonString = fileReader.readString();
                JSONObject jsonObject = JSONObject.parseObject(jsonString);
                Map<String, List<DailyData>> cityDataMap = new HashMap<>(16);
                for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
                    String key = entry.getKey();
                    Object value = entry.getValue();
                    if (value instanceof JSONArray) {
                        cityDataMap.put(key, ((JSONArray) value).toJavaList(DailyData.class));
                    }
                }
                for (String cityCode : cityDataMap.keySet()) {
                    if (!plantMap.containsKey(cityCode)) {
                        continue;
                    }
                    List<DailyData> dailyDataList = cityDataMap.get(cityCode);
                    List<BasePhotovoltaicPlantView> plantViewList = plantMap.get(cityCode);
                    //遍历预测数据列表
                    for (DailyData dailyData : dailyDataList) {
                        for (BasePhotovoltaicPlantView plant : plantViewList) {
                            List<PlantPredictedPowerDataEnt> allList = new ArrayList<>();
                            List<PlantPredictedPowerDataEnt> objList = new ArrayList<>();
                            DateTime dateTime = DateTime.of(dailyData.getDateTime());
                            int hours = 0;
                            String dayTime = dateTime.toString("yyyy-MM-dd");
                            for (int i = 0; i < 8; i++) {
                                String hourTime = dailyData.getHourTimeArray().get(i);
                                String plantId = plant.getId();
                                PlantPredictedPowerDataEnt dao = new PlantPredictedPowerDataEnt();
                                dao.setCreateTime(new Date());
                                dao.setPlantId(plantId);
                                dao.setHourTime(hourTime);
                                dao.setTemperature(new BigDecimal(StrUtil.replace(dailyData.getTemperatureArray().get(i), "℃", "")));
                                dao.setWindSpeed(new BigDecimal(StrUtil.replace(dailyData.getWindSpeedArray().get(i), "m/s", "")));
                                dao.setWindDirection(new BigDecimal(WindDirection.findByValue(dailyData.getWindDirectionArray().get(i)).getValue()));
                                dao.setPressure(new BigDecimal(StrUtil.replace(dailyData.getPressureArray().get(i), "hPa", "")));
                                dao.setHumidity(new BigDecimal(StrUtil.replace(dailyData.getHumidityArray().get(i), "%", "")));
                                //判断小时数与查到的小时数，如果查到的小时数小，判断为第二天
                                LocalTime localTime = LocalTime.parse(hourTime);
                                int hour = localTime.getHour();
                                if (hours >= hour) {
                                    dateTime = DateUtil.offsetDay(dateTime, 1);
                                    dayTime = dateTime.toString("yyyy-MM-dd");
                                }
                                dao.setYearTime(String.valueOf(dateTime.year()));
                                int month = dateTime.month() + 1;
                                dao.setMonthTime(this.int2Str(month));
                                int dayOfMonth = dateTime.dayOfMonth();
                                dao.setDayTime(this.int2Str(dayOfMonth));
                                hours = hour;
                                dao.setHourTime(this.int2Str(hours));
                                // 5.执行SQL
                                mapper.delete(new LambdaQueryWrapper<PlantPredictedPowerDataEnt>()
                                        .eq(PlantPredictedPowerDataEnt::getPlantId, plantId)
                                        .eq(PlantPredictedPowerDataEnt::getYearTime, dao.getYearTime())
                                        .eq(PlantPredictedPowerDataEnt::getMonthTime, dao.getMonthTime())
                                        .eq(PlantPredictedPowerDataEnt::getDayTime, dao.getDayTime())
                                        .eq(PlantPredictedPowerDataEnt::getHourTime, dao.getHourTime())
                                );
                                dao.setMinTime("00:00");
                                dao.setDataDate(new StringBuilder(dao.getYearTime())
                                        .append('-')
                                        .append(dao.getMonthTime())
                                        .append('-')
                                        .append(dao.getDayTime())
                                        .append(' ')
                                        .append(dao.getHourTime())
                                        .append(':')
                                        .append(dao.getMinTime()).toString());
                                PlantPredictedPowerDataEnt insertDao = XCopyUtils.copyNewObject(dao, PlantPredictedPowerDataEnt.class);
                                allList.add(insertDao);
                                PlantPredictedPowerDataEnt isExit = objList.stream().filter(item ->
                                                StrUtil.equals(item.getYearTime(), dao.getYearTime()) &&
                                                        StrUtil.equals(item.getMonthTime(), dao.getMonthTime()) &&
                                                        StrUtil.equals(item.getDayTime(), dao.getDayTime()) &&
                                                        StrUtil.equals(item.getHourTime(), dao.getHourTime()))
                                        .findFirst()
                                        .orElse(null);
                                if (Objects.isNull(isExit)) {
                                    objList.add(dao);
                                }
                            }
                            // 原站点时间结构为：08:00	11:00	14:00	17:00	20:00	23:00	02:00	05:00
                            // 两条数据中间差了两个小时，使用上下两条数据推算出中间差掉的两个小时


                        }


                    }
                }
            }
        } catch (MessagingException e) {
            throw new RuntimeException(e);
        } finally {
            IoUtil.close(folder);
            IoUtil.close(store);
        }
        return XServiceResult.OK;
    }

    /**
     * int转String
     *
     * @param num 数字
     * @return {@link String}
     */
    private String int2Str(int num) {
        return num < 10 ? "0" + num : String.valueOf(num);
    }


    /**
     * 保存附件
     *
     * @param part 部分
     * @return {@link String}
     * @throws MessagingException 消息传递异常
     * @throws IOException        IOException
     */
    private String saveAttachment(Part part) throws MessagingException, IOException {
        String fileName;
        String filePath = null;
        if (part.isMimeType("multipart/*")) {
            Multipart mp = (Multipart) part.getContent();
            BodyPart mPart;
            for (int i = 0; i < mp.getCount(); i++) {
                mPart = mp.getBodyPart(i);
                String disposition = mPart.getDisposition();
                if (Part.ATTACHMENT.equals(disposition) || Part.INLINE.equals(disposition)) {
                    fileName = mPart.getFileName();
                    if (StrUtil.isNotEmpty(fileName) && StringUtils.lowerCase(fileName).indexOf("GB2312") != -1) {
                        fileName = MimeUtility.decodeText(fileName);
                    }
                    fileName = StrUtil.replace(fileName, "\\?|=", "");
                    filePath = this.saveTempFile(fileName, mPart.getInputStream());
                } else if (mPart.isMimeType("multipart/*")) {
                    this.saveAttachment(mPart);
                } else {
                    fileName = mPart.getFileName();
                    if (StrUtil.isNotEmpty(fileName) && (StringUtils.lowerCase(fileName).indexOf("GB2312") != -1)) {
                        fileName = MimeUtility.decodeText(fileName);
                        filePath = this.saveTempFile(fileName, mPart.getInputStream());
                    }
                }
            }
        } else if (part.isMimeType("message/rfc822")) {
            this.saveAttachment((Part) part.getContent());
        }
        return filePath;
    }

    /**
     * 保存临时文件
     *
     * @param fileName 文件名
     * @param in       在里面
     * @return {@link String}
     */
    private String saveTempFile(String fileName, InputStream in) {
        String storeDir = XStartup.getCurrent().getProperty("weather.file.temp.path");
        StringBuilder stringBuilder = new StringBuilder(storeDir)
                .append(DateTime.now().toString("yyyyMMdd"));
        File savePath = new File(stringBuilder.toString());
        if (!savePath.exists()) {
            savePath.mkdirs();
        }
        String filePath = stringBuilder.append(File.separator).append(fileName).toString();
        FileUtil.writeFromStream(in, filePath);
        IoUtil.close(in);
        return filePath;
    }
}
